@@ -9,21 +9,21 @@
 // or else upgrading old project file will not work
 // reliably.
 // TODO QGIS 4: remove in favor of _QGIS_VERSION
-#define VERSION "3.12.0-București"
-#define _QGIS_VERSION "3.12.0-București"
+#define VERSION "3.14.16-Pi"
+#define _QGIS_VERSION "3.14.16-Pi"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 3120
+//#define VERSION_INT 31416
 // TODO QGIS 4: Remove in favor of _QGIS_VERSION_INT
-#define VERSION_INT 31200
-#define _QGIS_VERSION_INT 31200
-#define ABISYM(x) x ## 31200
+#define VERSION_INT 31416
+#define _QGIS_VERSION_INT 31416
+#define ABISYM(x) x ## 31416
 //used in main.cpp and anywhere else where the release name is needed
-#define RELEASE_NAME "București"
+#define RELEASE_NAME "Pi"
 
 #define QGIS_PLUGIN_SUBDIR "lib/qgis/plugins"
 #define QGIS_DATA_SUBDIR "share/qgis"
@@ -31,11 +31,11 @@
 #define QGIS_LIB_SUBDIR "lib"
 #define QGIS_QML_SUBDIR "qml"
 #define CMAKE_INSTALL_PREFIX "/usr"
-#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.12.0"
+#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.14.16"
 
 #define QGIS_SERVER_MODULE_SUBDIR "lib/qgis/server"
 
-#define QSCINTILLA_VERSION_STR "2.11.4"
+#define QSCINTILLA_VERSION_STR "2.11.5"
 
 #if defined( __APPLE__ )
 //used by Mac to find system or bundle resources relative to amount of bundling
@@ -43,7 +43,6 @@
 #endif
 
 #define QT_PLUGINS_DIR "/usr/lib/qt/plugins"
-#define OSG_PLUGINS_PATH ""
 
 /* #undef USING_NMAKE */
 
@@ -55,8 +54,6 @@
 
 /* #undef HAVE_ORACLE */
 
-/* #undef HAVE_OSGEARTHQT */
-
 /* #undef SERVER_SKIP_ECW */
 
 #define HAVE_SERVER_PYTHON_PLUGINS
@@ -67,6 +64,8 @@
 
 #define HAVE_3D
 
+#define HAVE_GEOREFERENCER
+
 #define USE_THREAD_LOCAL
 
 /* #undef QGISDEBUG */
