@@ -19,7 +19,7 @@
 #define QGSFEATURESINK_H
 
 #include "qgis_core.h"
-#include "qgis.h"
+#include "qgis_sip.h"
 #include "qgsfeature.h"
 #include "qgsfeatureiterator.h"
 
@@ -76,68 +76,32 @@ class CORE_EXPORT QgsFeatureSink
     /**
      * Adds a single \a feature to the sink. Feature addition behavior is controlled by the specified \a flags.
      * \see addFeatures()
-     * \returns true in case of success and false in case of failure
+     * \returns TRUE in case of success and FALSE in case of failure
      */
-    virtual bool addFeature( QgsFeature &feature, QgsFeatureSink::Flags flags = nullptr );
+    virtual bool addFeature( QgsFeature &feature, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() );
 
     /**
      * Adds a list of \a features to the sink. Feature addition behavior is controlled by the specified \a flags.
      * \see addFeature()
-     * \returns true in case of success and false in case of failure
+     * \returns TRUE in case of success and FALSE in case of failure
      */
-    virtual bool addFeatures( QgsFeatureList &features, QgsFeatureSink::Flags flags = nullptr ) = 0;
+    virtual bool addFeatures( QgsFeatureList &features, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) = 0;
 
     /**
      * Adds all features from the specified \a iterator to the sink. Feature addition behavior is controlled by the specified \a flags.
-     * \returns true if all features were added successfully, or false if any feature could not be added
+     * \returns TRUE if all features were added successfully, or FALSE if any feature could not be added
      */
-    virtual bool addFeatures( QgsFeatureIterator &iterator, QgsFeatureSink::Flags flags = nullptr );
+    virtual bool addFeatures( QgsFeatureIterator &iterator, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() );
 
     /**
      * Flushes any internal buffer which may exist in the sink, causing any buffered features to be added to the sink's destination.
-     * \returns false if any buffered features could not be added to the sink.
+     * \returns FALSE if any buffered features could not be added to the sink.
      */
     virtual bool flushBuffer() { return true; }
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsFeatureSink::Flags )
 
-
-/**
- * \class QgsProxyFeatureSink
- * \ingroup core
- * A simple feature sink which proxies feature addition on to another feature sink.
- *
- * This class is designed to allow factory methods which always return new QgsFeatureSink
- * objects. Since it is not always possible to create an entirely new QgsFeatureSink
- * (e.g. if the feature sink is a layer's data provider), a new QgsProxyFeatureSink
- * can instead be returned which forwards features on to the destination sink. The
- * proxy sink can be safely deleted without affecting the destination sink.
- *
- * \since QGIS 3.0
- */
-class CORE_EXPORT QgsProxyFeatureSink : public QgsFeatureSink
-{
-  public:
-
-    /**
-     * Constructs a new QgsProxyFeatureSink which forwards features onto a destination \a sink.
-     */
-    QgsProxyFeatureSink( QgsFeatureSink *sink );
-    bool addFeature( QgsFeature &feature, QgsFeatureSink::Flags flags = nullptr ) override { return mSink->addFeature( feature, flags ); }
-    bool addFeatures( QgsFeatureList &features, QgsFeatureSink::Flags flags = nullptr ) override { return mSink->addFeatures( features, flags ); }
-    bool addFeatures( QgsFeatureIterator &iterator, QgsFeatureSink::Flags flags = nullptr ) override { return mSink->addFeatures( iterator, flags ); }
-
-    /**
-     * Returns the destination QgsFeatureSink which the proxy will forward features to.
-     */
-    QgsFeatureSink *destinationSink() { return mSink; }
-
-  private:
-
-    QgsFeatureSink *mSink = nullptr;
-};
-
 Q_DECLARE_METATYPE( QgsFeatureSink * )
 
 #endif // QGSFEATURESINK_H
