@@ -17,7 +17,7 @@
 #define QGSFIELDS_H
 
 
-#include "qgis.h"
+#include "qgis_sip.h"
 #include "qgis_core.h"
 #include "qgsfield.h"
 
@@ -35,11 +35,13 @@ class QgsFieldsPrivate;
  * Container of fields for a vector layer.
  *
  * In addition to storing a list of QgsField instances, it also:
+ *
  * - allows quick lookups of field names to index in the list
  * - keeps track of where the field definition comes from (vector data provider, joined layer or newly added from an editing operation)
+ *
  * \note QgsFields objects are implicitly shared.
  */
-class CORE_EXPORT QgsFields
+class CORE_EXPORT  QgsFields
 {
   public:
 
@@ -94,16 +96,22 @@ class CORE_EXPORT QgsFields
 
     virtual ~QgsFields();
 
-    //! Remove all fields
+    //! Removes all fields
     void clear();
 
-    //! Append a field. The field must have unique name, otherwise it is rejected (returns false)
+    //! Appends a field. The field must have unique name, otherwise it is rejected (returns FALSE)
     bool append( const QgsField &field, FieldOrigin origin = OriginProvider, int originIndex = -1 );
 
-    //! Append an expression field. The field must have unique name, otherwise it is rejected (returns false)
+    /**
+     * Renames a name of field. The field must have unique name, otherwise change is rejected (returns FALSE)
+     * \since QGIS 3.6
+     */
+    bool rename( int fieldIdx, const QString &name );
+
+    //! Appends an expression field. The field must have unique name, otherwise it is rejected (returns FALSE)
     bool appendExpressionField( const QgsField &field, int originIndex );
 
-    //! Remove a field with the given index
+    //! Removes a field with the given index
     void remove( int fieldIdx );
 #ifdef SIP_RUN
     % MethodCode
@@ -119,10 +127,10 @@ class CORE_EXPORT QgsFields
     % End
 #endif
 
-    //! Extend with fields from another QgsFields container
+    //! Extends with fields from another QgsFields container
     void extend( const QgsFields &other );
 
-    //! Check whether the container is empty
+    //! Checks whether the container is empty
     bool isEmpty() const;
 
     //! Returns number of items
@@ -134,7 +142,7 @@ class CORE_EXPORT QgsFields
     sipRes = sipCpp->count();
     % End
 
-    //! Ensures that bool(obj) returns true (otherwise __len__() would be used)
+    //! Ensures that bool(obj) returns TRUE (otherwise __len__() would be used)
     int __bool__() const;
     % MethodCode
     sipRes = true;
@@ -153,7 +161,7 @@ class CORE_EXPORT QgsFields
     /**
      * Returns if a field index is valid
      * \param i  Index of the field which needs to be checked
-     * \returns   True if the field exists
+     * \returns   TRUE if the field exists
      */
     bool exists( int i ) const;
 
@@ -213,7 +221,7 @@ class CORE_EXPORT QgsFields
     int fieldIdx = sipCpp->indexFromName( *a0 );
     if ( fieldIdx == -1 )
     {
-      PyErr_SetString( PyExc_KeyError, a0->toAscii() );
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
       sipIsErr = 1;
     }
     else
@@ -282,7 +290,7 @@ class CORE_EXPORT QgsFields
     int indexOf( const QString &fieldName ) const;
 
     /**
-     * Look up field's index from the field name.
+     * Looks up field's index from the field name.
      * This method matches in the following order:
      *
      *  1. The exact field name taking case sensitivity into account
@@ -426,6 +434,8 @@ class CORE_EXPORT QgsFields
         inline const_iterator operator+( difference_type j ) const { return const_iterator( d + j ); }
         inline const_iterator operator-( difference_type j ) const { return const_iterator( d - j ); }
         inline int operator-( const_iterator j ) const { return int( d - j.d ); } // clazy:exclude=function-args-by-ref
+      private:
+        const_iterator &operator= ( const const_iterator & ) = delete;
     };
     friend class const_iterator;
     ///@endcond
