@@ -39,8 +39,14 @@
 #include "qgspointxy.h"
 #include "qgsmapunitscale.h"
 #include "qgsstringutils.h"
-#include "qgstextrenderer.h"
+#include "qgssymbol.h"
+#include "qgstextformat.h"
 #include "qgspropertycollection.h"
+#include "qgslabelobstaclesettings.h"
+#include "qgslabelthinningsettings.h"
+#include "qgslabeling.h"
+
+class QgsTextDocument;
 
 namespace pal SIP_SKIP
 {
@@ -70,7 +76,7 @@ class QgsVectorLayerLabelProvider;
 class QgsDxfExport;
 class QgsVectorLayerDiagramProvider;
 class QgsExpressionContext;
-
+class QgsCallout;
 
 /**
  * \ingroup core
@@ -96,9 +102,10 @@ class CORE_EXPORT QgsLabelPosition
      * \param pinned TRUE if label has pinned placement
      * \param providerId ID of associated label provider
      * \param labelGeometry polygon geometry of label boundary
+     * \param isUnplaced set to TRUE if label was unplaced (e.g. due to collisions with other labels)
      */
     QgsLabelPosition( QgsFeatureId id, double r, const QVector< QgsPointXY > &corners, const QgsRectangle &rect, double w, double h, const QString &layer, const QString &labeltext, const QFont &labelfont, bool upside_down, bool diagram = false, bool pinned = false, const QString &providerId = QString(),
-                      const QgsGeometry &labelGeometry = QgsGeometry() )
+                      const QgsGeometry &labelGeometry = QgsGeometry(), bool isUnplaced = false )
       : featureId( id )
       , rotation( r )
       , cornerPoints( corners )
@@ -113,6 +120,7 @@ class CORE_EXPORT QgsLabelPosition
       , isDiagram( diagram )
       , isPinned( pinned )
       , providerID( providerId )
+      , isUnplaced( isUnplaced )
     {}
 
     //! Constructor for QgsLabelPosition
@@ -182,8 +190,13 @@ class CORE_EXPORT QgsLabelPosition
      * \since QGIS 2.14
      */
     QString providerID;
-};
 
+    /**
+     * TRUE if label position corresponds to an unplaced label.
+     * \since QGIS 3.10
+     */
+    bool isUnplaced = false;
+};
 
 /**
  * \ingroup core
@@ -214,6 +227,7 @@ class CORE_EXPORT QgsPalLayerSettings
       Free, //!< Arranges candidates scattered throughout a polygon feature. Candidates are rotated to respect the polygon's orientation. Applies to polygon layers only.
       OrderedPositionsAroundPoint, //!< Candidates are placed in predefined positions around a point. Preference is given to positions with greatest cartographic appeal, e.g., top right, bottom right, etc. Applies to point layers only.
       PerimeterCurved, //!< Arranges candidates following the curvature of a polygon's boundary. Applies to polygon layers only.
+      OutsidePolygons, //!< Candidates are placed outside of polygon boundaries. Applies to polygon layers only. Since QGIS 3.14
     };
 
     //TODO QGIS 4.0 - move to QgsLabelingEngine
@@ -246,8 +260,7 @@ class CORE_EXPORT QgsPalLayerSettings
       FromSymbolBounds, //!< Offset distance applies from rendered symbol bounds
     };
 
-    //TODO QGIS 4.0 - move to QgsLabelingEngine, rename to LinePlacementFlag, use Q_DECLARE_FLAGS to make
-    //LinePlacementFlags type, and replace use of pal::LineArrangementFlag
+    //TODO QGIS 4.0 - remove, replaced by QgsLabeling::LinePlacementFlags
 
     /**
      * Line placement flags, which control how candidates are generated for a linear feature.
@@ -302,7 +315,7 @@ class CORE_EXPORT QgsPalLayerSettings
                                will be drawn with right alignment*/
     };
 
-    //TODO QGIS 4.0 - Move to QgsLabelingEngine
+    //TODO QGIS 4.0 - Remove -- moved to QgsLabelEngineObstacleSettings
 
     /**
      * Valid obstacle types, which affect how features within the layer will act as obstacles
@@ -345,6 +358,7 @@ class CORE_EXPORT QgsPalLayerSettings
       AutoWrapLength = 101,
       MultiLineHeight = 32,
       MultiLineAlignment = 33,
+      TextOrientation = 110,
       DirSymbDraw = 34,
       DirSymbLeft = 35,
       DirSymbRight = 36,
@@ -364,6 +378,13 @@ class CORE_EXPORT QgsPalLayerSettings
       BufferJoinStyle = 44,
       BufferBlendMode = 45,
 
+      // mask buffer
+      MaskEnabled = 104, //!< Whether the mask is enabled
+      MaskBufferSize = 105, //!< Mask buffer size
+      MaskBufferUnit = 106, //!< Mask buffer size unit
+      MaskOpacity = 107, //!< Mask opacity
+      MaskJoinStyle = 108, //!< Mask join style
+
       // background
       ShapeDraw = 46,
       ShapeKind = 47,
@@ -422,6 +443,9 @@ class CORE_EXPORT QgsPalLayerSettings
       Priority = 87,
       PredefinedPositionOrder = 91,
       LinePlacementOptions = 99, //!< Line placement flags
+      OverrunDistance = 102, //!< Distance which labels can extend past either end of linear features
+      LabelAllParts = 103, //!< Whether all parts of multi-part features should be labeled
+      PolygonLabelOutside = 109, //!< Whether labels outside a polygon feature are permitted, or should be forced (since QGIS 3.14)
 
       // rendering
       ScaleVisibility = 23,
@@ -435,6 +459,7 @@ class CORE_EXPORT QgsPalLayerSettings
       IsObstacle = 88,
       ObstacleFactor = 89,
       ZIndex = 90,
+      CalloutDraw = 100, //!< Show callout
 
       // (data defined only)
       Show = 15,
@@ -453,7 +478,32 @@ class CORE_EXPORT QgsPalLayerSettings
      *
      * \since QGIS 3.8
      */
-    bool prepare( const QgsRenderContext &context, QSet<QString> &attributeNames SIP_INOUT, const QgsFields &fields, const QgsMapSettings &mapSettings, const QgsCoordinateReferenceSystem &crs );
+    bool prepare( QgsRenderContext &context, QSet<QString> &attributeNames SIP_INOUT, const QgsFields &fields, const QgsMapSettings &mapSettings, const QgsCoordinateReferenceSystem &crs );
+
+    /**
+     * Returns all field names referenced by the configuration (e.g. field name or expression, data defined properties).
+     * \since QGIS 3.14
+     */
+    QSet<QString> referencedFields( const QgsRenderContext &context ) const;
+
+    /**
+     * Prepares the label settings for rendering.
+     *
+     * This should be called before rendering any labels, and must be
+     * followed by a call to stopRender() in order to gracefully clean up symbols.
+     *
+     * \since QGIS 3.10
+     */
+    void startRender( QgsRenderContext &context );
+
+    /**
+     * Finalises the label settings after use.
+     *
+     * This must be called after a call to startRender(), in order to gracefully clean up symbols.
+     *
+     * \since QGIS 3.10
+     */
+    void stopRender( QgsRenderContext &context );
 
     /**
      * Returns the labeling property definitions.
@@ -469,7 +519,7 @@ class CORE_EXPORT QgsPalLayerSettings
      * as an obstacle but having no labels of its own.
      * \since QGIS 2.12
      */
-    bool drawLabels;
+    bool drawLabels = true;
 
     //-- text style
 
@@ -484,19 +534,22 @@ class CORE_EXPORT QgsPalLayerSettings
      * TRUE if this label is made from a expression string, e.g., FieldName || 'mm'
      * \see fieldName
      */
-    bool isExpression;
+    bool isExpression = false;
 
     /**
      * Returns the QgsExpression for this label settings. May be NULLPTR if isExpression is FALSE.
      */
     QgsExpression *getLabelExpression();
 
-    QColor previewBkgrdColor;
+    /**
+     * \deprecated since QGIS 3.10. Use QgsTextFormat::previewBackgroundColor() instead.
+     */
+    Q_DECL_DEPRECATED QColor previewBkgrdColor = Qt::white;
 
     //! Substitution collection for automatic text substitution with labels
     QgsStringReplacementCollection substitutions;
     //! True if substitutions should be applied
-    bool useSubstitutions;
+    bool useSubstitutions = false;
 
     //-- text formatting
 
@@ -528,7 +581,7 @@ class CORE_EXPORT QgsPalLayerSettings
     bool useMaxLineLengthForAutoWrap = true;
 
     //! Horizontal alignment of multi-line labels.
-    MultiLineAlign multilineAlign;
+    MultiLineAlign multilineAlign = MultiFollowPlacement;
 
     /**
      * If TRUE, '<' or '>' (or custom strings set via leftDirectionSymbol and rightDirectionSymbol)
@@ -540,30 +593,30 @@ class CORE_EXPORT QgsPalLayerSettings
      * \see placeDirectionSymbol
      * \see reverseDirectionSymbol
      */
-    bool addDirectionSymbol;
+    bool addDirectionSymbol = false;
 
     /**
      * String to use for left direction arrows.
      * \see addDirectionSymbol
      * \see rightDirectionSymbol
      */
-    QString leftDirectionSymbol;
+    QString leftDirectionSymbol = QString( '<' );
 
     /**
      * String to use for right direction arrows.
      * \see addDirectionSymbol
      * \see leftDirectionSymbol
      */
-    QString rightDirectionSymbol;
+    QString rightDirectionSymbol = QString( '>' );
 
     /**
      * Placement option for direction symbols. Controls whether to place symbols to the left/right, above or below label.
      * \see addDirectionSymbol
      */
-    DirectionSymbols placeDirectionSymbol;
+    DirectionSymbols placeDirectionSymbol = SymbolLeftRight;
 
     //! True if direction symbols should be reversed
-    bool reverseDirectionSymbol;
+    bool reverseDirectionSymbol = false;
 
     /**
      * Set to TRUE to format numeric label text as numbers (e.g. inserting thousand separators
@@ -571,39 +624,60 @@ class CORE_EXPORT QgsPalLayerSettings
      * \see decimals
      * \see plusSign
      */
-    bool formatNumbers;
+    bool formatNumbers = false;
 
     /**
      * Number of decimal places to show for numeric labels. formatNumbers must be TRUE for this
      * setting to have an effect.
      * \see formatNumbers
      */
-    int decimals;
+    int decimals = 3;
 
     /**
      * Whether '+' signs should be prepended to positive numeric labels. formatNumbers must be TRUE for this
      * setting to have an effect.
      * \see formatNumbers
      */
-    bool plusSign;
+    bool plusSign = false;
 
     //-- placement
 
-    Placement placement;
+    Placement placement = AroundPoint;
+
+#ifndef SIP_RUN
+    unsigned int placementFlags = AboveLine | MapOrientation;
+#else
     unsigned int placementFlags;
+#endif
+
+    /**
+     * Returns the polygon placement flags, which dictate how polygon labels can be placed.
+     *
+     * \see setPolygonPlacementFlags()
+     * \since QGIS 3.14
+     */
+    QgsLabeling::PolygonPlacementFlags polygonPlacementFlags() const { return mPolygonPlacementFlags; }
+
+    /**
+     * Sets the polygon placement \a flags, which dictate how polygon labels can be placed.
+     *
+     * \see polygonPlacementFlags()
+     * \since QGIS 3.14
+     */
+    void setPolygonPlacementFlags( QgsLabeling::PolygonPlacementFlags flags ) { mPolygonPlacementFlags = flags; }
 
     /**
      * TRUE if feature centroid should be calculated from the whole feature, or
      * FALSE if only the visible part of the feature should be considered.
      */
-    bool centroidWhole;
+    bool centroidWhole = false;
 
     /**
      * TRUE if centroid positioned labels must be placed inside their corresponding
      * feature polygon, or FALSE if centroids which fall outside the polygon
      * are permitted.
      */
-    bool centroidInside;
+    bool centroidInside = false;
 
     /**
      * Ordered list of predefined label positions for points. Positions earlier
@@ -616,21 +690,21 @@ class CORE_EXPORT QgsPalLayerSettings
     /**
      * TRUE if only labels which completely fit within a polygon are allowed.
      */
-    bool fitInPolygonOnly;
+    bool fitInPolygonOnly = false;
 
     /**
      * Distance from feature to the label. Units are specified via distUnits.
      * \see distUnits
      * \see distMapUnitScale
      */
-    double dist;
+    double dist = 0;
 
     /**
      * Units the distance from feature to the label.
      * \see dist
      * \see distMapUnitScale
      */
-    QgsUnitTypes::RenderUnit distUnits;
+    QgsUnitTypes::RenderUnit distUnits = QgsUnitTypes::RenderMillimeters;
 
     /**
      * Map unit scale for label feature distance.
@@ -640,21 +714,21 @@ class CORE_EXPORT QgsPalLayerSettings
     QgsMapUnitScale distMapUnitScale;
 
     //! Offset type for layer (only applies in certain placement modes)
-    OffsetType offsetType;
+    OffsetType offsetType = FromPoint;
 
     /**
      * Distance for repeating labels for a single feature.
      * \see repeatDistanceUnit
      * \see repeatDistanceMapUnitScale
      */
-    double repeatDistance;
+    double repeatDistance = 0;
 
     /**
      * Units for repeating labels for a single feature.
      * \see repeatDistance
      * \see repeatDistanceMapUnitScale
      */
-    QgsUnitTypes::RenderUnit repeatDistanceUnit;
+    QgsUnitTypes::RenderUnit repeatDistanceUnit = QgsUnitTypes::RenderMillimeters;
 
     /**
      * Map unit scale for repeating labels for a single feature.
@@ -663,10 +737,34 @@ class CORE_EXPORT QgsPalLayerSettings
      */
     QgsMapUnitScale repeatDistanceMapUnitScale;
 
+    /**
+     * Distance which labels are allowed to overrun past the start or end of line features.
+     * \see overrunDistanceUnit
+     * \see repeatDistanceMapUnitScale
+     * \since QGIS 3.10
+     */
+    double overrunDistance = 0;
+
+    /**
+     * Units for label overrun distance.
+     * \see overrunDistance
+     * \see overrunDistanceMapUnitScale
+     * \since QGIS 3.10
+     */
+    QgsUnitTypes::RenderUnit overrunDistanceUnit = QgsUnitTypes::RenderMillimeters;
+
+    /**
+     * Map unit scale for label overrun distance.
+     * \see overrunDistance
+     * \see overrunDistanceUnit
+     * \since QGIS 3.10
+     */
+    QgsMapUnitScale overrunDistanceMapUnitScale;
+
     /**
      * Sets the quadrant in which to offset labels from feature.
      */
-    QuadrantPosition quadOffset;
+    QuadrantPosition quadOffset = QuadrantOver;
 
     /**
      * Horizontal offset of label. Units are specified via offsetUnits.
@@ -674,7 +772,7 @@ class CORE_EXPORT QgsPalLayerSettings
      * \see offsetUnits
      * \see labelOffsetMapUnitScale
      */
-    double xOffset;
+    double xOffset = 0;
 
     /**
      * Vertical offset of label. Units are specified via offsetUnits.
@@ -682,7 +780,7 @@ class CORE_EXPORT QgsPalLayerSettings
      * \see offsetUnits
      * \see labelOffsetMapUnitScale
      */
-    double yOffset;
+    double yOffset = 0;
 
     /**
      * Units for offsets of label.
@@ -690,7 +788,7 @@ class CORE_EXPORT QgsPalLayerSettings
      * \see yOffset
      * \see labelOffsetMapUnitScale
      */
-    QgsUnitTypes::RenderUnit offsetUnits;
+    QgsUnitTypes::RenderUnit offsetUnits = QgsUnitTypes::RenderMillimeters;
 
     /**
      * Map unit scale for label offset.
@@ -701,28 +799,28 @@ class CORE_EXPORT QgsPalLayerSettings
     QgsMapUnitScale labelOffsetMapUnitScale;
 
     //! Label rotation, in degrees clockwise
-    double angleOffset;
+    double angleOffset = 0;
 
     //! True if label rotation should be preserved during label pin/unpin operations.
-    bool preserveRotation;
+    bool preserveRotation = true;
 
     /**
      * Maximum angle between inside curved label characters (valid range 20.0 to 60.0).
      * \see maxCurvedCharAngleOut
      */
-    double maxCurvedCharAngleIn;
+    double maxCurvedCharAngleIn = 25.0;
 
     /**
      * Maximum angle between outside curved label characters (valid range -20.0 to -95.0)
      * \see maxCurvedCharAngleIn
      */
-    double maxCurvedCharAngleOut;
+    double maxCurvedCharAngleOut = -25.0;
 
     /**
      * Label priority. Valid ranges are from 0 to 10, where 0 = lowest priority
      * and 10 = highest priority.
      */
-    int priority;
+    int priority = 5;
 
     //-- rendering
 
@@ -731,7 +829,7 @@ class CORE_EXPORT QgsPalLayerSettings
      * \see maximumScale
      * \see minimumScale
      */
-    bool scaleVisibility;
+    bool scaleVisibility = false;
 
     /**
      * The maximum map scale (i.e. most "zoomed in" scale) at which the labels will be visible.
@@ -743,7 +841,7 @@ class CORE_EXPORT QgsPalLayerSettings
      * \see minimumScale
      * \see scaleVisibility
     */
-    double maximumScale;
+    double maximumScale = 0;
 
     /**
      * The minimum map scale (i.e. most "zoomed out" scale) at which the labels will be visible.
@@ -755,31 +853,31 @@ class CORE_EXPORT QgsPalLayerSettings
      * \see maximumScale
      * \see scaleVisibility
     */
-    double minimumScale;
+    double minimumScale = 0;
 
     /**
      * TRUE if label sizes should be limited by pixel size.
      * \see fontMinPixelSize
      * \see fontMaxPixelSize
      */
-    bool fontLimitPixelSize;
+    bool fontLimitPixelSize = false;
 
     /**
      * Minimum pixel size for showing rendered map unit labels (1 - 1000).
      * \see fontLimitPixelSize
      * \see fontMaxPixelSize
      */
-    int fontMinPixelSize;
+    int fontMinPixelSize = 0;
 
     /**
      * Maximum pixel size for showing rendered map unit labels (1 - 10000).
      * \see fontLimitPixelSize
      * \see fontMinPixelSize
      */
-    int fontMaxPixelSize;
+    int fontMaxPixelSize = 10000;
 
     //! If TRUE, all features will be labelled even when overlaps occur.
-    bool displayAll;
+    bool displayAll = false;
 
     //! Controls whether upside down labels are displayed and how they are handled.
     UpsideDownLabels upsidedownLabels = Upright;
@@ -788,56 +886,43 @@ class CORE_EXPORT QgsPalLayerSettings
      * TRUE if every part of a multi-part feature should be labeled. If FALSE,
      * only the largest part will be labeled.
      */
-    bool labelPerPart;
+    bool labelPerPart = false;
 
     /**
      * TRUE if connected line features with identical label text should be merged
      * prior to generating label positions.
      */
-    bool mergeLines;
+    bool mergeLines = false;
 
-    /**
-     * TRUE if the number of labels drawn should be limited.
-     * \see maxNumLabels
-     */
-    bool limitNumLabels;
+    // TODO QGIS 4.0 - remove this junk
 
-    /**
-     * The maximum number of labels which should be drawn for this layer.
-     * This only has an effect if limitNumLabels is TRUE.
-     * \see limitNumLabels
-     */
-    int maxNumLabels;
+#ifdef SIP_RUN
+    SIP_PROPERTY( name = limitNumLabels, get = _limitNumLabels, set = _setLimitNumLabels )
+    SIP_PROPERTY( name = maxNumLabels, get = _maxNumLabels, set = _setMaxNumLabels )
+    SIP_PROPERTY( name = minFeatureSize, get = _minFeatureSize, set = _setMinFeatureSize )
+    SIP_PROPERTY( name = obstacle, get = _getIsObstacle, set = _setIsObstacle )
+    SIP_PROPERTY( name = obstacleFactor, get = _getObstacleFactor, set = _setObstacleFactor )
+    SIP_PROPERTY( name = obstacleType, get = _getObstacleType, set = _setObstacleType )
+#endif
 
-    /**
-     * Minimum feature size (in millimeters) for a feature to be labelled.
-     */
-    double minFeatureSize;
+    ///@cond PRIVATE
+    bool _limitNumLabels() const { return mThinningSettings.limitNumberOfLabelsEnabled(); }
+    void _setLimitNumLabels( bool limit ) { mThinningSettings.setLimitNumberLabelsEnabled( limit ); }
+    int _maxNumLabels() const { return mThinningSettings.maximumNumberLabels(); }
+    void _setMaxNumLabels( int max ) { mThinningSettings.setMaximumNumberLabels( max ); }
+    double _minFeatureSize() const { return mThinningSettings.minimumFeatureSize(); }
+    void _setMinFeatureSize( double size ) { mThinningSettings.setMinimumFeatureSize( size ); }
+    bool _getIsObstacle() const { return mObstacleSettings.isObstacle(); }
+    void _setIsObstacle( bool obstacle ) { mObstacleSettings.setIsObstacle( obstacle ); }
+    double _getObstacleFactor() const { return mObstacleSettings.factor(); }
+    void _setObstacleFactor( double factor ) { mObstacleSettings.setFactor( factor ); }
+    ObstacleType _getObstacleType() const { return static_cast< ObstacleType>( mObstacleSettings.type() ); }
+    void _setObstacleType( ObstacleType type ) { mObstacleSettings.setType( static_cast< QgsLabelObstacleSettings::ObstacleType>( type ) ); }
 
-    /**
-     * TRUE if features for layer are obstacles to labels of other layers.
-     * \see obstacleFactor
-     * \see obstacleType
-     */
-    bool obstacle;
-
-    /**
-     * Obstacle factor, where 1.0 = default, < 1.0 more likely to be covered by labels,
-     * > 1.0 less likely to be covered
-     * \see obstacle
-     * \see obstacleType
-     */
-    double obstacleFactor;
-
-    /**
-     * Controls how features act as obstacles for labels.
-     * \see obstacle
-     * \see obstacleFactor
-     */
-    ObstacleType obstacleType;
+    ///@endcond
 
     //! Z-Index of label, where labels with a higher z-index are rendered on top of labels with a lower z-index
-    double zIndex;
+    double zIndex = 0;
 
     //! The geometry generator expression. Null if disabled.
     QString geometryGenerator;
@@ -848,11 +933,24 @@ class CORE_EXPORT QgsPalLayerSettings
     //! Defines if the geometry generator is enabled or not. If disabled, the standard geometry will be taken.
     bool geometryGeneratorEnabled = false;
 
+    /**
+     * Geometry type of layers associated with these settings.
+     * \since QGIS 3.10
+     */
+    QgsWkbTypes::GeometryType layerType = QgsWkbTypes::UnknownGeometry;
+
     /**
      * Calculates the space required to render the provided \a text in map units.
      * Results will be written to \a labelX and \a labelY.
+     * If the text orientation is set to rotation-based, the spaced taken to render
+     * vertically oriented text will be written to \a rotatedLabelX and \a rotatedLabelY .
      */
-    void calculateLabelSize( const QFontMetricsF *fm, const QString &text, double &labelX, double &labelY, const QgsFeature *f = nullptr, QgsRenderContext *context = nullptr );
+#ifndef SIP_RUN
+    void calculateLabelSize( const QFontMetricsF *fm, const QString &text, double &labelX, double &labelY, const QgsFeature *f = nullptr, QgsRenderContext *context = nullptr, double *rotatedLabelX SIP_OUT = nullptr, double *rotatedLabelY SIP_OUT = nullptr,
+                             QgsTextDocument *document = nullptr );
+#else
+    void calculateLabelSize( const QFontMetricsF *fm, const QString &text, double &labelX, double &labelY, const QgsFeature *f = nullptr, QgsRenderContext *context = nullptr, double *rotatedLabelX SIP_OUT = nullptr, double *rotatedLabelY SIP_OUT = nullptr );
+#endif
 
     /**
      * Register a feature for labeling.
@@ -866,22 +964,23 @@ class CORE_EXPORT QgsPalLayerSettings
      * symbol, the obstacle geometry should represent the bounds of the offset symbol). If not set,
      * the feature's original geometry will be used as an obstacle for labels. Not available
      * in Python bindings.
+     * \param symbol feature symbol to label (ownership is not transferred, and \a symbol must exist until the labeling is complete)
      */
     void registerFeature( const QgsFeature &f, QgsRenderContext &context,
                           QgsLabelFeature **labelFeature SIP_PYARGREMOVE = nullptr,
-                          QgsGeometry obstacleGeometry SIP_PYARGREMOVE = QgsGeometry() );
+                          QgsGeometry obstacleGeometry SIP_PYARGREMOVE = QgsGeometry(), const QgsSymbol *symbol SIP_PYARGREMOVE = nullptr );
 
     /**
      * Read settings from a DOM element
      * \since QGIS 2.12
      */
-    void readXml( QDomElement &elem, const QgsReadWriteContext &context );
+    void readXml( const QDomElement &elem, const QgsReadWriteContext &context );
 
     /**
      * Write settings into a DOM element
      * \since QGIS 2.12
      */
-    QDomElement writeXml( QDomDocument &doc, const QgsReadWriteContext &context );
+    QDomElement writeXml( QDomDocument &doc, const QgsReadWriteContext &context ) const;
 
     /**
      * Returns a reference to the label's property collection, used for data defined overrides.
@@ -923,10 +1022,84 @@ class CORE_EXPORT QgsPalLayerSettings
      */
     void setFormat( const QgsTextFormat &format ) { mFormat = format; }
 
+    /**
+     * Returns the label callout renderer, responsible for drawing label callouts.
+     *
+     * Ownership is not transferred.
+     *
+     * \see setCallout()
+     * \since QGIS 3.10
+     */
+    QgsCallout *callout() const { return mCallout.get(); }
+
+    /**
+     * Sets the label \a callout renderer, responsible for drawing label callouts.
+     *
+     * Ownership of \a callout is transferred to the settings.
+
+     * \see callout()
+     * \since QGIS 3.10
+     */
+    void setCallout( QgsCallout *callout SIP_TRANSFER );
+
+    /**
+     * Returns the label obstacle settings.
+     * \see setObstacleSettings()
+     * \note Not available in Python bindings
+     * \since QGIS 3.10.2
+     */
+    const QgsLabelObstacleSettings &obstacleSettings() const { return mObstacleSettings; } SIP_SKIP
+
+    /**
+     * Returns the label obstacle settings.
+     * \see setObstacleSettings()
+     * \since QGIS 3.10.2
+     */
+    QgsLabelObstacleSettings &obstacleSettings() { return mObstacleSettings; }
+
+    /**
+     * Sets the label obstacle \a settings.
+     * \see obstacleSettings()
+     * \since QGIS 3.10.2
+     */
+    void setObstacleSettings( const QgsLabelObstacleSettings &settings ) { mObstacleSettings = settings; }
+
+    /**
+     * Returns the label thinning settings.
+     * \see setThinningSettings()
+     * \note Not available in Python bindings
+     * \since QGIS 3.12
+     */
+    const QgsLabelThinningSettings &thinningSettings() const { return mThinningSettings; } SIP_SKIP
+
+    /**
+     * Returns the label thinning settings.
+     * \see setThinningSettings()
+     * \since QGIS 3.12
+     */
+    QgsLabelThinningSettings &thinningSettings() { return mThinningSettings; }
+
+    /**
+     * Sets the label thinning \a settings.
+     * \see thinningSettings()
+     * \since QGIS 3.12
+     */
+    void setThinningSettings( const QgsLabelThinningSettings &settings ) { mThinningSettings = settings; }
+
+    /**
+    * Returns a pixmap preview for label \a settings.
+    * \param settings label settings
+    * \param size target pixmap size
+    * \param previewText text to render in preview, or empty for default text
+    * \param padding space between icon edge and color ramp
+    * \since QGIS 3.10
+    */
+    static QPixmap labelSettingsPreviewPixmap( const QgsPalLayerSettings &settings, QSize size, const QString &previewText = QString(), int padding = 0 );
+
     // temporary stuff: set when layer gets prepared or labeled
     const QgsFeature *mCurFeat = nullptr;
     QgsFields mCurFields;
-    int fieldIndex;
+    int fieldIndex = 0;
     const QgsMapToPixel *xform = nullptr;
     QgsCoordinateTransform ct;
 
@@ -985,6 +1158,8 @@ class CORE_EXPORT QgsPalLayerSettings
 
     void parseTextBuffer( QgsRenderContext &context );
 
+    void parseTextMask( QgsRenderContext &context );
+
     void parseTextFormatting( QgsRenderContext &context );
 
     void parseShapeBackground( QgsRenderContext &context );
@@ -1012,15 +1187,18 @@ class CORE_EXPORT QgsPalLayerSettings
 
     QgsTextFormat mFormat;
 
+    std::unique_ptr< QgsCallout > mCallout;
+
+    QgsLabelObstacleSettings mObstacleSettings;
+    QgsLabelThinningSettings mThinningSettings;
+
+    QgsLabeling::PolygonPlacementFlags mPolygonPlacementFlags = QgsLabeling::PolygonPlacementFlag::AllowPlacementInsideOfPolygon;
+
     QgsExpression mGeometryGeneratorExpression;
 
-    static const QVector< PredefinedPointPosition > DEFAULT_PLACEMENT_ORDER;
+    bool mRenderStarted = false;
 
     static void initPropertyDefinitions();
-
-    //! Property definitions
-    static QgsPropertiesDefinition sPropertyDefinitions;
-
 };
 
 /**
@@ -1086,10 +1264,10 @@ class CORE_EXPORT QgsPalLabeling
   public:
 
     /**
-     * called to find out whether the layer is used for labeling
+     * Called to find out whether a specified \a layer is used for labeling.
      * \since QGIS 2.4
      */
-    static bool staticWillUseLayer( QgsVectorLayer *layer );
+    static bool staticWillUseLayer( const QgsMapLayer *layer );
 
     //! \note not available in Python bindings
     static void drawLabelCandidateRect( pal::LabelPosition *lp, QPainter *painter, const QgsMapToPixel *xform, QList<QgsLabelCandidate> *candidates = nullptr ) SIP_SKIP;
@@ -1155,6 +1333,10 @@ class CORE_EXPORT QgsPalLabeling
     static void dataDefinedTextBuffer( QgsPalLayerSettings &tmpLyr,
                                        const QMap< QgsPalLayerSettings::Property, QVariant > &ddValues );
 
+    //! Update temporary QgsPalLayerSettings with any data defined mask values
+    static void dataDefinedTextMask( QgsPalLayerSettings &tmpLyr,
+                                     const QMap< QgsPalLayerSettings::Property, QVariant > &ddValues );
+
     //! Update temporary QgsPalLayerSettings with any data defined shape background values
     static void dataDefinedShapeBackground( QgsPalLayerSettings &tmpLyr,
                                             const QMap< QgsPalLayerSettings::Property, QVariant > &ddValues );
@@ -1179,4 +1361,5 @@ class CORE_EXPORT QgsPalLabeling
     friend class QgsPalLayerSettings;
 };
 
+
 #endif // QGSPALLABELING_H
