@@ -21,16 +21,22 @@
 #include "qgis_core.h"
 
 #include "qgsrasterlayer.h"
-#include "qgsvectorlayer.h"
 #include "qgsmessagelog.h"
 #include "qgsspatialindex.h"
 #include "qgsprocessing.h"
+#include "qgsfeaturesink.h"
+#include "qgsfeaturesource.h"
+#include "qgsproxyfeaturesink.h"
+#include "qgsremappingproxyfeaturesink.h"
 
+class QgsMeshLayer;
 class QgsProject;
 class QgsProcessingContext;
 class QgsMapLayerStore;
 class QgsProcessingFeedback;
 class QgsProcessingFeatureSource;
+class QgsProcessingAlgorithm;
+class QgsVectorTileLayer;
 
 #include <QString>
 #include <QVariant>
@@ -43,16 +49,16 @@ class QgsProcessingFeatureSource;
  */
 class CORE_EXPORT QgsProcessingUtils
 {
-
   public:
 
     /**
      * Returns a list of raster layers from a \a project which are compatible with the processing
      * framework.
      *
-     * If the \a sort argument is true then the layers will be sorted by their QgsMapLayer::name()
+     * If the \a sort argument is TRUE then the layers will be sorted by their QgsMapLayer::name()
      * value.
      * \see compatibleVectorLayers()
+     * \see compatibleMeshLayers()
      * \see compatibleLayers()
      */
     static QList< QgsRasterLayer * > compatibleRasterLayers( QgsProject *project, bool sort = true );
@@ -66,37 +72,88 @@ class CORE_EXPORT QgsProcessingUtils
      * layers with the specified source type included in the list will be returned. Leaving the \a sourceTypes
      * list empty will cause all vector layers, regardless of their geometry type, to be returned.
      *
-     * If the \a sort argument is true then the layers will be sorted by their QgsMapLayer::name()
+     * If the \a sort argument is TRUE then the layers will be sorted by their QgsMapLayer::name()
      * value.
      * \see compatibleRasterLayers()
+     * \see compatibleMeshLayers()
      * \see compatibleLayers()
      */
     static QList< QgsVectorLayer * > compatibleVectorLayers( QgsProject *project,
         const QList< int > &sourceTypes = QList< int >(),
         bool sort = true );
 
+    /**
+     * Returns a list of mesh layers from a \a project which are compatible with the processing
+     * framework.
+     *
+     * If the \a sort argument is TRUE then the layers will be sorted by their QgsMapLayer::name()
+     * value.
+     *
+     * \see compatibleRasterLayers()
+     * \see compatibleVectorLayers()
+     * \see compatibleLayers()
+     *
+     * \since QGIS 3.6
+     */
+    static QList<QgsMeshLayer *> compatibleMeshLayers( QgsProject *project, bool sort = true );
+
     /**
      * Returns a list of map layers from a \a project which are compatible with the processing
      * framework.
      *
-     * If the \a sort argument is true then the layers will be sorted by their QgsMapLayer::name()
+     * If the \a sort argument is TRUE then the layers will be sorted by their QgsMapLayer::name()
      * value.
      * \see compatibleRasterLayers()
      * \see compatibleVectorLayers()
      */
     static QList< QgsMapLayer * > compatibleLayers( QgsProject *project, bool sort = true );
 
+    /**
+     * Encodes a provider key and layer \a uri to a single string, for use with
+     * decodeProviderKeyAndUri()
+     *
+     * \since QGIS 3.14
+     */
+    static QString encodeProviderKeyAndUri( const QString &providerKey, const QString &uri );
+
+    /**
+     * Decodes a provider key and layer \a uri from an encoded string, for use with
+     * encodeProviderKeyAndUri()
+     *
+     * \param string encoded string, as returned by encodeProviderKeyAndUri()
+     * \param providerKey ID key for corresponding data provider
+     * \param uri decoded layer uri
+     * \returns TRUE if \a string was successfully decoded
+     *
+     * \since QGIS 3.14
+     */
+    static bool decodeProviderKeyAndUri( const QString &string, QString &providerKey SIP_OUT, QString &uri SIP_OUT );
+
+    /**
+     * Layer type hints.
+     * \since QGIS 3.4
+     */
+    enum class LayerHint SIP_MONKEYPATCH_SCOPEENUM : int
+    {
+      UnknownType, //!< Unknown layer type
+      Vector, //!< Vector layer type
+      Raster, //!< Raster layer type
+      Mesh, //!< Mesh layer type, since QGIS 3.6
+    };
+
     /**
      * Interprets a string as a map layer within the supplied \a context.
      *
      * The method will attempt to
      * load a layer matching the passed \a string. E.g. if the string matches a layer ID or name
      * within the context's project or temporary layer store then this layer will be returned.
-     * If the string is a file path and \a allowLoadingNewLayers is true, then the layer at this
+     * If the string is a file path and \a allowLoadingNewLayers is TRUE, then the layer at this
      * file path will be loaded and added to the context's temporary layer store.
      * Ownership of the layer remains with the \a context or the context's current project.
+     *
+     * The \a typeHint can be used to dictate the type of map layer expected.
      */
-    static QgsMapLayer *mapLayerFromString( const QString &string, QgsProcessingContext &context, bool allowLoadingNewLayers = true );
+    static QgsMapLayer *mapLayerFromString( const QString &string, QgsProcessingContext &context, bool allowLoadingNewLayers = true, QgsProcessingUtils::LayerHint typeHint = QgsProcessingUtils::LayerHint::UnknownType );
 
     /**
      * Converts a variant \a value to a new feature source.
@@ -111,14 +168,34 @@ class CORE_EXPORT QgsProcessingUtils
      */
     static QgsProcessingFeatureSource *variantToSource( const QVariant &value, QgsProcessingContext &context, const QVariant &fallbackValue = QVariant() ) SIP_FACTORY;
 
+    /**
+     * Converts a variant \a value to a coordinate reference system.
+     *
+     * The optional \a fallbackValue can be used to specify a "default" value which is used
+     * if \a value cannot be successfully converted to a CRS.
+     *
+     * \since QGIS 3.12
+     */
+    static QgsCoordinateReferenceSystem variantToCrs( const QVariant &value, QgsProcessingContext &context, const QVariant &fallbackValue = QVariant() );
+
     /**
      * Normalizes a layer \a source string for safe comparison across different
      * operating system environments.
      */
     static QString normalizeLayerSource( const QString &source );
 
+    /**
+     * Converts a variant to a Python literal.
+     *
+     * \see stringToPythonLiteral()
+     * \since QGSIS 3.6
+     */
+    static QString variantToPythonLiteral( const QVariant &value );
+
     /**
      * Converts a string to a Python string literal. E.g. by replacing ' with \'.
+     *
+     * \see variantToPythonLiteral()
      */
     static QString stringToPythonLiteral( const QString &string );
 
@@ -140,13 +217,14 @@ class CORE_EXPORT QgsProcessingUtils
      * The caller takes responsibility for deleting the returned sink.
      */
 #ifndef SIP_RUN
-    static QgsFeatureSink *createFeatureSink(
-      QString &destination,
-      QgsProcessingContext &context,
-      const QgsFields &fields,
-      QgsWkbTypes::Type geometryType,
-      const QgsCoordinateReferenceSystem &crs,
-      const QVariantMap &createOptions = QVariantMap() ) SIP_FACTORY;
+    static QgsFeatureSink *createFeatureSink( QString &destination,
+        QgsProcessingContext &context,
+        const QgsFields &fields,
+        QgsWkbTypes::Type geometryType,
+        const QgsCoordinateReferenceSystem &crs,
+        const QVariantMap &createOptions = QVariantMap(),
+        QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags(),
+        QgsRemappingSinkDefinition *remappingDefinition = nullptr ) SIP_FACTORY;
 #endif
 
     /**
@@ -170,11 +248,21 @@ class CORE_EXPORT QgsProcessingUtils
      */
     static void createFeatureSinkPython( QgsFeatureSink **sink SIP_OUT SIP_TRANSFERBACK, QString &destination SIP_INOUT, QgsProcessingContext &context, const QgsFields &fields, QgsWkbTypes::Type geometryType, const QgsCoordinateReferenceSystem &crs, const QVariantMap &createOptions = QVariantMap() ) SIP_THROW( QgsProcessingException ) SIP_PYNAME( createFeatureSink );
 
+
+    /**
+     * Combines the extent of several map \a layers. If specified, the target \a crs
+     * will be used to transform the layer's extent to the desired output reference system
+     * using the specified \a context.
+     * \since QGIS 3.8
+     */
+    static QgsRectangle combineLayerExtents( const QList<QgsMapLayer *> &layers, const QgsCoordinateReferenceSystem &crs, QgsProcessingContext &context );
+
     /**
      * Combines the extent of several map \a layers. If specified, the target \a crs
      * will be used to transform the layer's extent to the desired output reference system.
+     * \deprecated Use version with QgsProcessingContext argument instead
      */
-    static QgsRectangle combineLayerExtents( const QList<QgsMapLayer *> &layers, const QgsCoordinateReferenceSystem &crs = QgsCoordinateReferenceSystem() );
+    Q_DECL_DEPRECATED static QgsRectangle combineLayerExtents( const QList<QgsMapLayer *> &layers, const QgsCoordinateReferenceSystem &crs = QgsCoordinateReferenceSystem() ) SIP_DEPRECATED;
 
     /**
      * Converts an \a input parameter value for use in source iterating mode, where one individual sink
@@ -205,7 +293,7 @@ class CORE_EXPORT QgsProcessingUtils
     static QString formatHelpMapAsHtml( const QVariantMap &map, const QgsProcessingAlgorithm *algorithm );
 
     /**
-     * Converts a source vector \a layer to a file path to a vector layer of compatible format.
+     * Converts a source vector \a layer to a file path of a vector layer of compatible format.
      *
      * If the specified \a layer is not of the format listed in the
      * \a compatibleFormats argument, then the layer will first be exported to a compatible format
@@ -215,6 +303,13 @@ class CORE_EXPORT QgsProcessingUtils
      *
      * The \a preferredFormat argument is used to specify to desired file extension to use when a temporary
      * layer export is required. This defaults to shapefiles.
+     *
+     * The \a featureLimit argument can be used to specify a limit on the number of features read from the layer.
+     *
+     * When an algorithm is capable of handling multi-layer input files (such as Geopackage), it is preferable
+     * to use convertToCompatibleFormatAndLayerName() which may avoid conversion in more situations.
+     *
+     * \see convertToCompatibleFormatAndLayerName()
      */
     static QString convertToCompatibleFormat( const QgsVectorLayer *layer,
         bool selectedFeaturesOnly,
@@ -222,7 +317,49 @@ class CORE_EXPORT QgsProcessingUtils
         const QStringList &compatibleFormats,
         const QString &preferredFormat,
         QgsProcessingContext &context,
-        QgsProcessingFeedback *feedback );
+        QgsProcessingFeedback *feedback, long long featureLimit = -1 );
+
+    /**
+     * Converts a source vector \a layer to a file path and layer name of a vector layer of compatible format.
+     *
+     * If the specified \a layer is not of the format listed in the
+     * \a compatibleFormats argument, then the layer will first be exported to a compatible format
+     * in a temporary location using \a baseName. The function will then return the path to that temporary file.
+     *
+     * \a compatibleFormats should consist entirely of lowercase file extensions, e.g. 'shp'.
+     *
+     * The \a featureLimit argument can be used to specify a limit on the number of features read from the layer.
+     *
+     * The \a preferredFormat argument is used to specify to desired file extension to use when a temporary
+     * layer export is required. This defaults to shapefiles.
+     *
+     * This method should be preferred over convertToCompatibleFormat() when an algorithm is able
+     * to correctly handle files with multiple layers. Unlike convertToCompatibleFormat(), it will not force
+     * a conversion in this case and will return the target layer name in the \a layerName argument.
+     *
+     * \param layer source layer to convert (if required)
+     * \param selectedFeaturesOnly TRUE if only selected features from the layer should be used
+     * \param baseName base file name for converted layer, if required
+     * \param compatibleFormats a list of lowercase file extensions compatible with the algorithm
+     * \param preferredFormat preferred format extension to use if conversion if required
+     * \param context processing context
+     * \param feedback feedback object
+     * \param layerName will be set to the target layer name for multi-layer sources (e.g. Geopackage)
+     * \param featureLimit can be used to place a limit on the maximum number of features read from the layer
+     *
+     * \returns path to source layer, or nearly converted compatible layer
+     *
+     * \see convertToCompatibleFormat()
+     * \since QGIS 3.10
+     */
+    static QString convertToCompatibleFormatAndLayerName( const QgsVectorLayer *layer,
+        bool selectedFeaturesOnly,
+        const QString &baseName,
+        const QStringList &compatibleFormats,
+        const QString &preferredFormat,
+        QgsProcessingContext &context,
+        QgsProcessingFeedback *feedback,
+        QString &layerName SIP_OUT, long long featureLimit = -1 );
 
     /**
      * Combines two field lists, avoiding duplicate field names (in a case-insensitive manner).
@@ -234,7 +371,7 @@ class CORE_EXPORT QgsProcessingUtils
      * length of field names, so be aware that the results of calling this method may
      * be truncated when saving to these formats.
      */
-    static QgsFields combineFields( const QgsFields &fieldsA, const QgsFields &fieldsB );
+    static QgsFields combineFields( const QgsFields &fieldsA, const QgsFields &fieldsB, const QString &fieldsBPrefix = QString() );
 
     /**
      * Returns a list of field indices parsed from the given list of field names. Unknown field names are ignored.
@@ -249,9 +386,34 @@ class CORE_EXPORT QgsProcessingUtils
      */
     static QgsFields indicesToFields( const QList<int> &indices, const QgsFields &fields );
 
-  private:
+    /**
+     * Returns the default vector extension to use, in the absence of all other constraints (e.g.
+     * provider based support for extensions).
+     *
+     * This method returns the user-set default extension from the processing settings, or
+     * a fallback value of "gpkg".
+     *
+     * \see defaultRasterExtension()
+     * \since QGIS 3.10
+     */
+    static QString defaultVectorExtension();
 
+    /**
+     * Returns the default raster extension to use, in the absence of all other constraints (e.g.
+     * provider based support for extensions).
+     *
+     * This method returns the user-set default extension from the processing settings, or
+     * a fallback value of "tif".
+     *
+     * \see defaultVectorExtension()
+     * \since QGIS 3.10
+     */
+    static QString defaultRasterExtension();
+
+  private:
     static bool canUseLayer( const QgsRasterLayer *layer );
+    static bool canUseLayer( const QgsMeshLayer *layer );
+    static bool canUseLayer( const QgsVectorTileLayer *layer );
     static bool canUseLayer( const QgsVectorLayer *layer,
                              const QList< int > &sourceTypes = QList< int >() );
 
@@ -265,19 +427,34 @@ class CORE_EXPORT QgsProcessingUtils
      * returned.
      * \see mapLayerFromString()
      */
-    static QgsMapLayer *mapLayerFromStore( const QString &string, QgsMapLayerStore *store );
+    static QgsMapLayer *mapLayerFromStore( const QString &string, QgsMapLayerStore *store, QgsProcessingUtils::LayerHint typeHint = QgsProcessingUtils::LayerHint::UnknownType );
+
+    /**
+     * Interprets a string as a map layer. The method will attempt to
+     * load a layer matching the passed \a string using the given coordinate
+     * \a transformContext.
+     * E.g. if the string is a file path,
+     * then the layer at this file path will be loaded.
+     * The caller takes responsibility for deleting the returned map layer.
+     *
+     * \since QGIS 3.8
+     */
+    static QgsMapLayer *loadMapLayerFromString( const QString &string, const QgsCoordinateTransformContext &transformContext, LayerHint typeHint = LayerHint::UnknownType );
 
     /**
      * Interprets a string as a map layer. The method will attempt to
      * load a layer matching the passed \a string. E.g. if the string is a file path,
      * then the layer at this file path will be loaded.
      * The caller takes responsibility for deleting the returned map layer.
+     *
+     * \deprecated use mapLayerFromString() that takes QgsCoordinateTransformContext as an argument instead
      */
-    static QgsMapLayer *loadMapLayerFromString( const QString &string );
+    Q_DECL_DEPRECATED static QgsMapLayer *loadMapLayerFromString( const QString &string, LayerHint typeHint = LayerHint::UnknownType ) SIP_DEPRECATED ;
 
-    static void parseDestinationString( QString &destination, QString &providerKey, QString &uri, QString &layerName, QString &format, QMap<QString, QVariant> &options, bool &useWriter );
+    static void parseDestinationString( QString &destination, QString &providerKey, QString &uri, QString &layerName, QString &format, QMap<QString, QVariant> &options, bool &useWriter, QString &extension );
 
     friend class TestQgsProcessing;
+    friend class QgsProcessingProvider;
 
 };
 
@@ -302,11 +479,15 @@ class CORE_EXPORT QgsProcessingFeatureSource : public QgsFeatureSource
     /**
      * Constructor for QgsProcessingFeatureSource, accepting an original feature source \a originalSource
      * and processing \a context.
-     * Ownership of \a originalSource is dictated by \a ownsOriginalSource. If \a ownsOriginalSource is false,
+     * Ownership of \a originalSource is dictated by \a ownsOriginalSource. If \a ownsOriginalSource is FALSE,
      * ownership is not transferred, and callers must ensure that \a originalSource exists for the lifetime of this object.
-     * If \a ownsOriginalSource is true, then this object will take ownership of \a originalSource.
+     * If \a ownsOriginalSource is TRUE, then this object will take ownership of \a originalSource.
+     *
+     * If \a featureLimit is set to a value > 0, then a limit is placed on the maximum number of features which will be
+     * read from the source.
      */
-    QgsProcessingFeatureSource( QgsFeatureSource *originalSource, const QgsProcessingContext &context, bool ownsOriginalSource = false );
+    QgsProcessingFeatureSource( QgsFeatureSource *originalSource, const QgsProcessingContext &context, bool ownsOriginalSource = false,
+                                long long featureLimit = -1 );
 
     ~QgsProcessingFeatureSource() override;
 
@@ -317,6 +498,8 @@ class CORE_EXPORT QgsProcessingFeatureSource : public QgsFeatureSource
      */
     QgsFeatureIterator getFeatures( const QgsFeatureRequest &request, Flags flags ) const;
 
+    QgsFeatureSource::FeatureAvailability hasFeatures() const override;
+
     QgsFeatureIterator getFeatures( const QgsFeatureRequest &request = QgsFeatureRequest() ) const override;
     QgsCoordinateReferenceSystem sourceCrs() const override;
     QgsFields fields() const override;
@@ -328,12 +511,20 @@ class CORE_EXPORT QgsProcessingFeatureSource : public QgsFeatureSource
     QVariant maximumValue( int fieldIndex ) const override;
     QgsRectangle sourceExtent() const override;
     QgsFeatureIds allFeatureIds() const override;
+    SpatialIndexPresence hasSpatialIndex() const override;
 
     /**
      * Returns an expression context scope suitable for this source.
      */
     QgsExpressionContextScope *createExpressionContextScope() const SIP_FACTORY;
 
+    /**
+     * Overrides the default geometry check method for the source.
+     *
+     * \since QGIS 3.14
+     */
+    void setInvalidGeometryCheck( QgsFeatureRequest::InvalidGeometryCheck method );
+
   private:
 
     QgsFeatureSource *mSource = nullptr;
@@ -342,6 +533,11 @@ class CORE_EXPORT QgsProcessingFeatureSource : public QgsFeatureSource
     std::function< void( const QgsFeature & ) > mInvalidGeometryCallback;
     std::function< void( const QgsFeature & ) > mTransformErrorCallback;
 
+    std::function< void( const QgsFeature & ) > mInvalidGeometryCallbackSkip;
+    std::function< void( const QgsFeature & ) > mInvalidGeometryCallbackAbort;
+
+    long long mFeatureLimit = -1;
+
 };
 
 #ifndef SIP_RUN
@@ -367,15 +563,15 @@ class CORE_EXPORT QgsProcessingFeatureSink : public QgsProxyFeatureSink
      *
      * The \a sinkName is used to identify the destination sink when reporting errors.
      *
-     * Ownership of \a originalSink is dictated by \a ownsOriginalSource. If \a ownsOriginalSink is false,
+     * Ownership of \a originalSink is dictated by \a ownsOriginalSource. If \a ownsOriginalSink is FALSE,
      * ownership is not transferred, and callers must ensure that \a originalSink exists for the lifetime of this object.
-     * If \a ownsOriginalSink is true, then this object will take ownership of \a originalSink.
+     * If \a ownsOriginalSink is TRUE, then this object will take ownership of \a originalSink.
      */
     QgsProcessingFeatureSink( QgsFeatureSink *originalSink, const QString &sinkName, QgsProcessingContext &context, bool ownsOriginalSink = false );
     ~QgsProcessingFeatureSink() override;
-    bool addFeature( QgsFeature &feature, QgsFeatureSink::Flags flags = nullptr ) override;
-    bool addFeatures( QgsFeatureList &features, QgsFeatureSink::Flags flags = nullptr ) override;
-    bool addFeatures( QgsFeatureIterator &iterator, QgsFeatureSink::Flags flags = nullptr ) override;
+    bool addFeature( QgsFeature &feature, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) override;
+    bool addFeatures( QgsFeatureList &features, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) override;
+    bool addFeatures( QgsFeatureIterator &iterator, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) override;
 
   private:
 
