@@ -79,6 +79,8 @@ class GUI_EXPORT QgsRendererWidget : public QgsPanelWidget
      */
     void applyChanges();
 
+    void setDockMode( bool dockMode ) override;
+
   signals:
 
     /**
@@ -100,6 +102,18 @@ class GUI_EXPORT QgsRendererWidget : public QgsPanelWidget
     QAction *mCopyAction = nullptr;
     QAction *mPasteAction = nullptr;
 
+    /**
+     * Copy symbol action.
+     * \since QGIS 3.10
+     */
+    QAction *mCopySymbolAction = nullptr;
+
+    /**
+     * Paste symbol action.
+     * \since QGIS 3.10
+     */
+    QAction *mPasteSymbolAction = nullptr;
+
     //! Context in which widget is shown
     QgsSymbolWidgetContext mContext;
 
@@ -131,9 +145,21 @@ class GUI_EXPORT QgsRendererWidget : public QgsPanelWidget
     //! Change marker angles of selected symbols
     void changeSymbolAngle();
 
+
     virtual void copy() {}
     virtual void paste() {}
 
+    /**
+      * Pastes the clipboard symbol over selected items.
+      *
+      * \since QGIS 3.10
+     */
+    virtual void pasteSymbolToSelection();
+
+  private slots:
+
+    void copySymbol();
+
   private:
 
     /**
