@@ -13,8 +13,8 @@
  *                                                                         *
  ***************************************************************************/
 
-#ifndef QGSRULEBASEDRENDERERV2_H
-#define QGSRULEBASEDRENDERERV2_H
+#ifndef QGSRULEBASEDRENDERER_H
+#define QGSRULEBASEDRENDERER_H
 
 #include "qgis_core.h"
 #include "qgis_sip.h"
@@ -45,10 +45,13 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
       FeatDrawMarkers = 2
     };
 
-    // feature for rendering: QgsFeature and some flags
+    /**
+     * Feature for rendering by a QgsRuleBasedRenderer. Contains a QgsFeature and some flags.
+     * \ingroup core
+     */
     struct FeatureToRender
     {
-      FeatureToRender( QgsFeature &_f, int _flags )
+      FeatureToRender( const QgsFeature &_f, int _flags )
         : feat( _f )
         , flags( _flags )
       {}
@@ -56,25 +59,40 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
       int flags; // selected and/or draw markers
     };
 
-    // rendering job: a feature to be rendered with a particular symbol
-    // (both f, symbol are _not_ owned by this class)
+    /**
+     * A QgsRuleBasedRenderer rendering job, consisting of a feature to be rendered with a particular symbol.
+     * \ingroup core
+     */
     struct RenderJob
     {
-      RenderJob( QgsRuleBasedRenderer::FeatureToRender &_ftr, QgsSymbol *_s )
-        : ftr( _ftr )
-        , symbol( _s )
-      {}
-      QgsRuleBasedRenderer::FeatureToRender &ftr;
-      QgsSymbol *symbol = nullptr;
+        RenderJob( QgsRuleBasedRenderer::FeatureToRender &_ftr, QgsSymbol *_s )
+          : ftr( _ftr )
+          , symbol( _s )
+        {}
+
+        //! Feature to render
+        QgsRuleBasedRenderer::FeatureToRender &ftr;
+
+        //! Symbol to render feature with (not owned by this object).
+        QgsSymbol *symbol = nullptr;
+
+      private:
+#ifdef SIP_RUN
+        RenderJob &operator=( const RenderJob & );
+#endif
     };
 
-    // render level: a list of jobs to be drawn at particular level
-    // (jobs are owned by this class)
+    /**
+     * Render level: a list of jobs to be drawn at particular level for a QgsRuleBasedRenderer.
+     * \ingroup core
+     */
     struct RenderLevel
     {
       explicit RenderLevel( int z ): zIndex( z ) {}
-      ~RenderLevel() { Q_FOREACH ( RenderJob *j, jobs ) delete j; }
+      ~RenderLevel() { qDeleteAll( jobs ); }
       int zIndex;
+
+      //! List of jobs to render, owned by this object.
       QList<QgsRuleBasedRenderer::RenderJob *> jobs;
 
       QgsRuleBasedRenderer::RenderLevel &operator=( const QgsRuleBasedRenderer::RenderLevel &rh )
@@ -82,7 +100,7 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
         zIndex = rh.zIndex;
         qDeleteAll( jobs );
         jobs.clear();
-        Q_FOREACH ( RenderJob *job, rh.jobs )
+        for ( RenderJob *job :  qgis::as_const( rh.jobs ) )
         {
           jobs << new RenderJob( *job );
         }
@@ -90,9 +108,9 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
       }
 
       RenderLevel( const QgsRuleBasedRenderer::RenderLevel &other )
-        : zIndex( other.zIndex )
+        : zIndex( other.zIndex ), jobs()
       {
-        Q_FOREACH ( RenderJob *job, other.jobs )
+        for ( RenderJob *job : qgis::as_const( other.jobs ) )
         {
           jobs << new RenderJob( *job );
         }
@@ -100,7 +118,7 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
 
     };
 
-    // rendering queue: a list of rendering levels
+    //! Rendering queue: a list of rendering levels
     typedef QList<QgsRuleBasedRenderer::RenderLevel> RenderQueue;
 
     class Rule;
@@ -144,13 +162,13 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
         QString dump( int indent = 0 ) const;
 
         /**
-         * Return the attributes used to evaluate the expression of this rule
+         * Returns the attributes used to evaluate the expression of this rule
          * \returns A set of attribute names
          */
         QSet<QString> usedAttributes( const QgsRenderContext &context ) const;
 
         /**
-         * Returns true if this rule or one of its chilren needs the geometry to be applied.
+         * Returns TRUE if this rule or one of its children needs the geometry to be applied.
          */
         bool needsGeometry() const;
 
@@ -165,20 +183,20 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
          *
          * \param f         The feature to test
          * \param context   The context in which the rendering happens
-         * \returns          True if the feature shall be rendered
+         * \returns          TRUE if the feature shall be rendered
          */
-        bool isFilterOK( QgsFeature &f, QgsRenderContext *context = nullptr ) const;
+        bool isFilterOK( const QgsFeature &f, QgsRenderContext *context = nullptr ) const;
 
         /**
          * Check if this rule applies for a given \a scale.
          * The \a scale value indicates the scale denominator, e.g. 1000.0 for a 1:1000 map.
-         * If set to 0, it will always return true.
+         * If set to 0, it will always return TRUE.
          *
          * \returns If the rule will be evaluated at this scale
          */
         bool isScaleOK( double scale ) const;
 
-        QgsSymbol *symbol() { return mSymbol; }
+        QgsSymbol *symbol() { return mSymbol.get(); }
         QString label() const { return mLabel; }
         bool dependsOnScale() const { return mMaximumScale != 0 || mMinimumScale != 0; }
 
@@ -206,7 +224,7 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
          * A filter that will check if this rule applies
          * \returns An expression
          */
-        QgsExpression *filter() const { return mFilter; }
+        QgsExpression *filter() const { return mFilter.get(); }
 
         /**
          * A filter that will check if this rule applies
@@ -224,7 +242,7 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
         /**
          * Returns if this rule is active
          *
-         * \returns True if the rule is active
+         * \returns TRUE if the rule is active
          */
         bool active() const { return mIsActive; }
 
@@ -240,7 +258,7 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
          */
         void setRuleKey( const QString &key ) { mRuleKey = key; }
 
-        //! set a new symbol (or NULL). Deletes old symbol.
+        //! Sets a new symbol (or NULLPTR). Deletes old symbol.
         void setSymbol( QgsSymbol *sym SIP_TRANSFER );
         void setLabel( const QString &label ) { mLabel = label; }
 
@@ -297,7 +315,7 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
         //! prepare the rule for rendering and its children (build active children array)
         bool startRender( QgsRenderContext &context, const QgsFields &fields, QString &filter );
 
-        //! get all used z-levels from this rule and children
+        //! Gets all used z-levels from this rule and children
         QSet<int> collectZLevels();
 
         /**
@@ -318,19 +336,26 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
         QgsRuleBasedRenderer::Rule::RenderResult renderFeature( QgsRuleBasedRenderer::FeatureToRender &featToRender, QgsRenderContext &context, QgsRuleBasedRenderer::RenderQueue &renderQueue );
 
         //! only tell whether a feature will be rendered without actually rendering it
-        bool willRenderFeature( QgsFeature &feat, QgsRenderContext *context = nullptr );
+        bool willRenderFeature( const QgsFeature &feature, QgsRenderContext *context = nullptr );
 
         //! tell which symbols will be used to render the feature
-        QgsSymbolList symbolsForFeature( QgsFeature &feat, QgsRenderContext *context = nullptr );
+        QgsSymbolList symbolsForFeature( const QgsFeature &feature, QgsRenderContext *context = nullptr );
 
         /**
          * Returns which legend keys match the feature
          * \since QGIS 2.14
          */
-        QSet< QString > legendKeysForFeature( QgsFeature &feat, QgsRenderContext *context = nullptr );
+        QSet< QString > legendKeysForFeature( const QgsFeature &feature, QgsRenderContext *context = nullptr );
 
-        //! tell which rules will be used to render the feature
-        QgsRuleBasedRenderer::RuleList rulesForFeature( QgsFeature &feat, QgsRenderContext *context = nullptr );
+        /**
+         * Returns the list of rules used to render the feature in a specific
+         * context.
+         *
+         * \param feature The feature for which rules have to be find
+         * \param context The rendering context
+         * \param onlyActive TRUE to search for active rules only, FALSE otherwise
+         */
+        QgsRuleBasedRenderer::RuleList rulesForFeature( const QgsFeature &feature, QgsRenderContext *context = nullptr, bool onlyActive = true );
 
         /**
          * Stop a rendering process. Used to clean up the internal state of this rule
@@ -350,18 +375,18 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
         static QgsRuleBasedRenderer::Rule *create( QDomElement &ruleElem, QgsSymbolMap &symbolMap ) SIP_FACTORY;
 
         /**
-         * Return all children rules of this rule
+         * Returns all children rules of this rule
          *
          * \returns A list of rules
          */
-        QgsRuleBasedRenderer::RuleList &children() { return mChildren; }
+        const QgsRuleBasedRenderer::RuleList &children() { return mChildren; }
 
         /**
          * Returns all children, grand-children, grand-grand-children, grand-gra... you get it
          *
          * \returns A list of descendant rules
          */
-        QgsRuleBasedRenderer::RuleList descendants() const { RuleList l; Q_FOREACH ( QgsRuleBasedRenderer::Rule *c, mChildren ) { l += c; l += c->descendants(); } return l; }
+        QgsRuleBasedRenderer::RuleList descendants() const;
 
         /**
          * The parent rule
@@ -382,10 +407,10 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
         //! delete child rule
         void removeChildAt( int i );
 
-        //! take child rule out, set parent as null
+        //! take child rule out, set parent as NULLPTR
         QgsRuleBasedRenderer::Rule *takeChild( QgsRuleBasedRenderer::Rule *rule ) SIP_TRANSFERBACK;
 
-        //! take child rule out, set parent as null
+        //! take child rule out, set parent as NULLPTR
         QgsRuleBasedRenderer::Rule *takeChildAt( int i ) SIP_TRANSFERBACK;
 
         /**
@@ -397,43 +422,54 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
         /**
          * Sets if this rule is an ELSE rule
          *
-         * \param iselse If true, this rule is an ELSE rule
+         * \param iselse If TRUE, this rule is an ELSE rule
          */
         void setIsElse( bool iselse );
 
         /**
          * Check if this rule is an ELSE rule
          *
-         * \returns True if this rule is an else rule
+         * \returns TRUE if this rule is an else rule
          */
-        bool isElse() { return mElseRule; }
+        bool isElse() const { return mElseRule; }
+
+        /**
+         * Accepts the specified symbology \a visitor, causing it to visit all child rules associated
+         * with the rule.
+         *
+         * Returns TRUE if the visitor should continue visiting other objects, or FALSE if visiting
+         * should be canceled.
+         *
+         * \since QGIS 3.10
+         */
+        bool accept( QgsStyleEntityVisitorInterface *visitor ) const;
 
       protected:
         void initFilter();
 
-        Rule *mParent; // parent rule (NULL only for root rule)
-        QgsSymbol *mSymbol = nullptr;
-        double mMaximumScale = 0;
-        double mMinimumScale = 0;
-        QString mFilterExp, mLabel, mDescription;
-        bool mElseRule;
-        RuleList mChildren;
-        RuleList mElseRules;
-        bool mIsActive; // whether it is enabled or not
-
-        QString mRuleKey; // string used for unique identification of rule within renderer
-
-        // temporary
-        QgsExpression *mFilter = nullptr;
-        // temporary while rendering
-        QSet<int> mSymbolNormZLevels;
-        RuleList mActiveChildren;
-
       private:
 #ifdef SIP_RUN
         Rule( const QgsRuleBasedRenderer::Rule &rh );
 #endif
 
+        Rule *mParent = nullptr; // parent rule (nullptr only for root rule)
+        std::unique_ptr< QgsSymbol > mSymbol;
+        double mMaximumScale = 0;
+        double mMinimumScale = 0;
+        QString mFilterExp, mLabel, mDescription;
+        bool mElseRule = false;
+        RuleList mChildren;
+        RuleList mElseRules;
+        bool mIsActive = true; // whether it is enabled or not
+
+        QString mRuleKey; // string used for unique identification of rule within renderer
+
+        // temporary
+        std::unique_ptr< QgsExpression > mFilter;
+        // temporary while rendering
+        QSet<int> mSymbolNormZLevels;
+        RuleList mActiveChildren;
+
         /**
          * Check which child rules are else rules and update the internal list of else rules
          *
@@ -453,10 +489,10 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
 
     ~QgsRuleBasedRenderer() override;
 
-    //! return symbol for current feature. Should not be used individually: there could be more symbols for a feature
-    QgsSymbol *symbolForFeature( QgsFeature &feature, QgsRenderContext &context ) override;
+    //! Returns symbol for current feature. Should not be used individually: there could be more symbols for a feature
+    QgsSymbol *symbolForFeature( const QgsFeature &feature, QgsRenderContext &context ) const override;
 
-    bool renderFeature( QgsFeature &feature, QgsRenderContext &context, int layer = -1, bool selected = false, bool drawVertexMarker = false ) override;
+    bool renderFeature( const QgsFeature &feature, QgsRenderContext &context, int layer = -1, bool selected = false, bool drawVertexMarker = false ) override SIP_THROW( QgsCsException );
 
     void startRender( QgsRenderContext &context, const QgsFields &fields ) override;
 
@@ -474,7 +510,7 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
 
     static QgsFeatureRenderer *createFromSld( QDomElement &element, QgsWkbTypes::GeometryType geomType ) SIP_FACTORY;
 
-    QgsSymbolList symbols( QgsRenderContext &context ) override;
+    QgsSymbolList symbols( QgsRenderContext &context ) const override;
 
     QDomElement save( QDomDocument &doc, const QgsReadWriteContext &context ) override;
     bool legendSymbolItemsCheckable() const override;
@@ -484,11 +520,12 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
     void setLegendSymbolItem( const QString &key, QgsSymbol *symbol SIP_TRANSFER ) override;
     QgsLegendSymbolList legendSymbolItems() const override;
     QString dump() const override;
-    bool willRenderFeature( QgsFeature &feat, QgsRenderContext &context ) override;
-    QgsSymbolList symbolsForFeature( QgsFeature &feat, QgsRenderContext &context ) override;
-    QgsSymbolList originalSymbolsForFeature( QgsFeature &feat, QgsRenderContext &context ) override;
-    QSet<QString> legendKeysForFeature( QgsFeature &feature, QgsRenderContext &context ) override;
+    bool willRenderFeature( const QgsFeature &feature, QgsRenderContext &context ) const override;
+    QgsSymbolList symbolsForFeature( const QgsFeature &feature, QgsRenderContext &context ) const override;
+    QgsSymbolList originalSymbolsForFeature( const QgsFeature &feature, QgsRenderContext &context ) const override;
+    QSet<QString> legendKeysForFeature( const QgsFeature &feature, QgsRenderContext &context ) const override;
     QgsFeatureRenderer::Capabilities capabilities() override { return MoreSymbolsPerFeature | Filter | ScaleDependent; }
+    bool accept( QgsStyleEntityVisitorInterface *visitor ) const override;
 
     /////
 
@@ -530,4 +567,4 @@ class CORE_EXPORT QgsRuleBasedRenderer : public QgsFeatureRenderer
 #endif
 };
 
-#endif // QGSRULEBASEDRENDERERV2_H
+#endif // QGSRULEBASEDRENDERER_H
