@@ -39,9 +39,10 @@ class CORE_EXPORT QgsValueRelationFieldFormatter : public QgsFieldFormatter
     struct ValueRelationItem
     {
       //! Constructor for ValueRelationItem
-      ValueRelationItem( const QVariant &key, const QString &value )
+      ValueRelationItem( const QVariant &key, const QString &value, const QString &description = QString() )
         : key( key )
         , value( value )
+        , description( description )
       {}
 
       //! Constructor for ValueRelationItem
@@ -49,6 +50,7 @@ class CORE_EXPORT QgsValueRelationFieldFormatter : public QgsFieldFormatter
 
       QVariant key;
       QString value;
+      QString description;
     };
 
     typedef QVector < QgsValueRelationFieldFormatter::ValueRelationItem > ValueRelationCache;
@@ -77,11 +79,12 @@ class CORE_EXPORT QgsValueRelationFieldFormatter : public QgsFieldFormatter
      * if doing multiple lookups in a loop.
      * \param config The widget configuration
      * \param formFeature The feature currently being edited with current attribute values
+     * \param parentFormFeature For embedded forms only, the feature currently being edited in the parent form with current attribute values
      * \return A kvp list of values for the widget
      *
      * \since QGIS 3.0
      */
-    static QgsValueRelationFieldFormatter::ValueRelationCache createCache( const QVariantMap &config, const QgsFeature &formFeature = QgsFeature() );
+    static QgsValueRelationFieldFormatter::ValueRelationCache createCache( const QVariantMap &config, const QgsFeature &formFeature = QgsFeature(), const QgsFeature &parentFormFeature = QgsFeature() );
 
     /**
      * Check if the \a expression requires a form scope (i.e. if it uses fields
@@ -112,12 +115,42 @@ class CORE_EXPORT QgsValueRelationFieldFormatter : public QgsFieldFormatter
     static QSet<QString> expressionFormVariables( const QString &expression );
 
     /**
-     * Check whether the \a feature has all values required by the \a expression
+     * Check if the \a expression requires a parent form scope (i.e. if it uses fields
+     * or geometry of the parent form's currently edited feature).
+     *
+     * \param expression The widget's filter expression
+     * \return TRUE if the expression requires a parent form scope
+     * \since QGIS 3.14
+     */
+    static bool expressionRequiresParentFormScope( const QString &expression );
+
+    /**
+     * Returns a list of attributes required by the parent form's form context \a expression
+     *
+     * \param expression Form filter expression
+     * \return list of parent attributes required by the expression
+     * \since QGIS 3.14
+     */
+    static QSet<QString> expressionParentFormAttributes( const QString &expression );
+
+    /**
+     * Returns a list of variables required by the parent form's form context \a expression
+     *
+     * \param expression Form filter expression
+     * \return list of parent variables required by the expression
+     * \since QGIS 3.14
+     */
+    static QSet<QString> expressionParentFormVariables( const QString &expression );
+
+
+    /**
+     * Check whether the \a feature has all values required by the \a expression,
+     * optionally checks for \a parentFeature
      *
      * \return TRUE if the expression can be used
      * \since QGIS 3.2
      */
-    static bool expressionIsUsable( const QString &expression, const QgsFeature &feature );
+    static bool expressionIsUsable( const QString &expression, const QgsFeature &feature, const QgsFeature &parentFeature = QgsFeature() );
 
     /**
      * Returns the (possibly NULL) layer from the widget's \a config and \a project
