@@ -21,10 +21,6 @@ __author__ = 'Hugo Mercier'
 __date__ = 'January 2016'
 __copyright__ = '(C) 2016, Hugo Mercier'
 
-# This will get replaced with a git SHA1 when you do a git archive323
-
-__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
-
 from qgis.core import (QgsVirtualLayerDefinition,
                        QgsVectorLayer,
                        QgsWkbTypes,
@@ -32,12 +28,15 @@ from qgis.core import (QgsVirtualLayerDefinition,
                        QgsProcessingParameterMultipleLayers,
                        QgsProcessingParameterDefinition,
                        QgsExpression,
+                       QgsProcessingUtils,
                        QgsProcessingParameterString,
                        QgsProcessingParameterEnum,
                        QgsProcessingParameterCrs,
                        QgsProcessingParameterFeatureSink,
                        QgsFeatureSink,
-                       QgsProcessingException)
+                       QgsProcessingException,
+                       QgsVectorFileWriter,
+                       QgsProject)
 
 from processing.algs.qgis.QgisAlgorithm import QgisAlgorithm
 
@@ -58,7 +57,6 @@ class ParameterExecuteSql(QgsProcessingParameterDefinition):
 
 
 class ExecuteSQL(QgisAlgorithm):
-
     """ This algorithm allows executing an SQL query on a set of input
     vector layers thanks to the virtual layer provider
     """
@@ -129,7 +127,20 @@ class ExecuteSQL(QgisAlgorithm):
 
         df = QgsVirtualLayerDefinition()
         for layerIdx, layer in enumerate(layers):
-            df.addSource('input{}'.format(layerIdx + 1), layer.id())
+
+            # Issue https://github.com/qgis/QGIS/issues/24041
+            # When using this algorithm from the graphic modeler, it may try to
+            # access (thanks the QgsVirtualLayerProvider) to memory layer that
+            # belongs to temporary QgsMapLayerStore, not project.
+            # So, we write them to disk is this is the case.
+            if not context.project().mapLayer(layer.id()):
+                basename = "memorylayer." + QgsVectorFileWriter.supportedFormatExtensions()[0]
+                tmp_path = QgsProcessingUtils.generateTempFilename(basename)
+                QgsVectorFileWriter.writeAsVectorFormat(
+                    layer, tmp_path, layer.dataProvider().encoding())
+                df.addSource('input{}'.format(layerIdx + 1), tmp_path, "ogr")
+            else:
+                df.addSource('input{}'.format(layerIdx + 1), layer.id())
 
         if query == '':
             raise QgsProcessingException(
@@ -156,6 +167,9 @@ class ExecuteSQL(QgisAlgorithm):
         if not vLayer.isValid():
             raise QgsProcessingException(vLayer.dataProvider().error().message())
 
+        if vLayer.wkbType() == QgsWkbTypes.Unknown:
+            raise QgsProcessingException(self.tr("Cannot find geometry field"))
+
         (sink, dest_id) = self.parameterAsSink(parameters, self.OUTPUT, context,
                                                vLayer.fields(), vLayer.wkbType() if geometry_type != 1 else 1, vLayer.crs())
         if sink is None:
